unit VisibilityD7_IMPL;

interface

uses
  SysUtils, ComObj, ComServ, ActiveX, Variants, Outlook2000, Office2000, adxAddIn, VisibilityD7_TLB, StdVcl,
  Classes, adxolFormsManager, Unit1, Unit2, uUtils;

type
  TCoVisibilityD7 = class(TadxAddin, ICoVisibilityD7)
  end;

  TAddInModule = class(TadxCOMAddInModule)
    adxOlFormsManager1: TadxOlFormsManager;
    adxOlExplorerCommandbar1: TadxOlExplorerCommandbar;
    adxOlInspectorCommandBar1: TadxOlInspectorCommandBar;
    adxRibbonTab1: TadxRibbonTab;
    adxRibbonTab2: TadxRibbonTab;
    procedure adxOlExplorerCommandbar1Controls0Change(Sender: TObject);
    procedure adxOlExplorerCommandbar1Controls1Click(Sender: TObject);
    procedure adxRibbonTab1Controls0Controls0Click(Sender: TObject;
      const RibbonControl: IRibbonControl);
    procedure adxRibbonTab2Controls0Controls0Click(Sender: TObject;
      const RibbonControl: IRibbonControl);
    procedure adxCOMAddInModuleCreate(Sender: TObject);
    procedure adxCOMAddInModuleAddInBeginShutdown(Sender: TObject);
    procedure adxCOMAddInModuleError(const E: Exception;
      var Handled: Boolean);
    procedure adxOlFormsManager1Error(ASender: TObject;
      Args: TadxOlErrorEventArgs);
    procedure adxOlFormsManager1ADXBeforeFormInstanceCreateEx(
      ASender: TObject; Args: TBeforeFormInstanceCreateEventArgs);
    procedure adxOlFormsManager1Initialize(ASender: TObject);
    procedure adxOlInspectorCommandBar1Controls0Click(Sender: TObject);
  private
    FHiddenExplorerForm: TadxOlForm1;
    FExplorerMode: ExplorerFormDisplayMode;
    folders: OutlookFolders;
    function GetExplorerComboBox: TadxCommandBarComboBox;
    function GetHiddenExplorerForm: TadxOlForm1;
    procedure SetHiddenExplorerForm(const Value: TadxOlForm1);
    function GetInspectorCommandBarButton: TadxCommandBarButton;
    function GetInspectorCommandBar: TadxOlInspectorCommandBar;
    function GetRestoreCommandBarButton: TadxCommandBarButton;
    procedure SetExplorerFormDisplayMode(ExplorerMode: ExplorerFormDisplayMode; currentFolder: MAPIFolder);
    function GetExplorerFormsCollectionItem: TadxOlFormsCollectionItem;
    function GetInspectorFormsCollectionItem: TadxOlFormsCollectionItem;
    procedure ProcessInspectorButtonClick;
  private
    explorerMode: ExplorerFormDisplayMode;

    procedure InitializeExplorerCommandBarComboBox();
    property ExplorerComboBox: TadxCommandBarComboBox read GetExplorerComboBox;
  protected
  public
    ShowByInspectorCommandBarButtonClick: boolean;
    property HiddenExplorerForm: TadxOlForm1 read GetHiddenExplorerForm write SetHiddenExplorerForm;
    property InspectorCommandBarButton: TadxCommandBarButton read GetInspectorCommandBarButton;
    property InspectorCommandBar: TadxOlInspectorCommandBar read GetInspectorCommandBar;
    property RestoreCommandBarButton: TadxCommandBarButton read GetRestoreCommandBarButton;
    property ExplorerFormsCollectionItem: TadxOlFormsCollectionItem read GetExplorerFormsCollectionItem;
    property InspectorFormsCollectionItem: TadxOlFormsCollectionItem read GetInspectorFormsCollectionItem;

  end;

implementation


{$R *.dfm}

procedure TAddInModule.adxOlExplorerCommandbar1Controls0Change(
  Sender: TObject);
var
  activeExplorer: Explorer;
  currentFolder: MAPIFolder;
  newExplorerMode: ExplorerFormDisplayMode;
begin
  activeExplorer := nil;
  currentFolder := nil;
  try
    activeExplorer := OutlookApp.ActiveExplorer();
    if (activeExplorer <> nil) then begin
      currentFolder := activeExplorer.CurrentFolder;
    end;

    newExplorerMode := efdmUnknown;

    if (ExplorerComboBox.Text = Strings.InboxOnlyItem) then begin
      newExplorerMode := efdmInboxOnlyItem;
    end;

    if (ExplorerComboBox.Text = Strings.InboxAndSubfoldersItem) then begin
      newExplorerMode := efdmInboxAndSubfoldersItem;
    end;

    if (newExplorerMode <> explorerMode) then begin
      SetExplorerFormDisplayMode(newExplorerMode, currentFolder);
      adxOlFormsManager1.Items[1].ApplyTo(activeExplorer);
    end;
  finally
    activeExplorer := nil;
    currentFolder := nil;
  end;
end;

procedure TAddInModule.adxOlExplorerCommandbar1Controls1Click(
  Sender: TObject);
begin
  if (HiddenExplorerForm <> nil) then begin
    (HiddenExplorerForm as TadxOlForm1).ShowAndClearHideIndication();
    HiddenExplorerForm := nil;
  end;
end;

procedure TAddInModule.adxRibbonTab1Controls0Controls0Click(
  Sender: TObject; const RibbonControl: IRibbonControl);
begin
  ProcessInspectorButtonClick();
end;

procedure TAddInModule.adxRibbonTab2Controls0Controls0Click(
  Sender: TObject; const RibbonControl: IRibbonControl);
begin
  ProcessInspectorButtonClick();
end;

procedure TAddInModule.adxCOMAddInModuleCreate(Sender: TObject);
begin
  folders := nil;
  explorerMode := efdmUnknown;
  ShowByInspectorCommandBarButtonClick := false;
  InitializeExplorerCommandBarComboBox();
end;

procedure TAddInModule.InitializeExplorerCommandBarComboBox;
begin
  ExplorerComboBox.Items.Clear;
  ExplorerComboBox.Items.Add(Strings.InboxOnlyItem);
  ExplorerComboBox.Items.Add(Strings.InboxAndSubfoldersItem);
  ExplorerComboBox.ListIndex := 1;
end;


function TAddInModule.GetExplorerComboBox: TadxCommandBarComboBox;
begin
  result := nil;
  if adxOlExplorerCommandbar1.Controls.Count > 0 then begin
    result := adxOlExplorerCommandbar1.Controls[0] as TadxCommandBarComboBox;
  end;
end;

function TAddInModule.GetHiddenExplorerForm: TadxOlForm1;
begin
  result := FHiddenExplorerForm; 
end;

procedure TAddInModule.SetHiddenExplorerForm(const Value: TadxOlForm1);
begin
  FHiddenExplorerForm := Value;
  if (FHiddenExplorerForm = nil) then begin
    RestoreCommandBarButton.Visible := false;
  end
  else begin
    RestoreCommandBarButton.Visible := true;
  end;

end;

function TAddInModule.GetInspectorCommandBarButton: TadxCommandBarButton;
begin
  result := adxOlInspectorCommandBar1.Controls[0] as TadxCommandBarButton;
end;

function TAddInModule.GetInspectorCommandBar: TadxOlInspectorCommandBar;
begin
  result := adxOlInspectorCommandBar1;
end;

function TAddInModule.GetRestoreCommandBarButton: TadxCommandBarButton;
begin
  result := adxOlExplorerCommandbar1.Controls[1] as TadxCommandBarButton;
end;

procedure TAddInModule.SetExplorerFormDisplayMode(
  ExplorerMode: ExplorerFormDisplayMode; currentFolder: MAPIFolder);
var
  TheCurrentFolderIsInboxFolder: boolean;
  visibleForm: TadxOlForm;
  activeExplorer: Explorer;
begin
  if (FExplorerMode <> ExplorerMode) then begin
    case (ExplorerMode) of
      efdmInboxOnlyItem: begin
        //Hide a visible form when current folder is not the Inbox folder.
        TheCurrentFolderIsInboxFolder := folders.IsInboxFolder(currentFolder);
        if (not TheCurrentFolderIsInboxFolder) then begin
            visibleForm := ADXUtils.GetVisibleForm(ExplorerFormsCollectionItem);
            if (visibleForm <> nil) then begin
                visibleForm.Hide();
            end;
        end;

        //Note that all invisible cached forms will be deleted.
        ExplorerFormsCollectionItem.FolderName := folders.InboxFullFolderName;
        FExplorerMode := ExplorerMode;
      end;

      //See the ADXOlFormsManager.ADXBeforeFormInstanceCreate event for the code that works in the InboxAndSubfoldersItem mode.
      efdmInboxAndSubfoldersItem: begin
        //Note that all invisible cached forms will be deleted.
        ExplorerFormsCollectionItem.FolderName := Strings.AllFolders;
        FExplorerMode := ExplorerMode;

        activeExplorer := OutlookApp.ActiveExplorer();
        try
          ExplorerFormsCollectionItem.ApplyTo(activeExplorer);
        finally
          activeExplorer := nil;
        end;
      end;
    end;
  end;
end;

function TAddInModule.GetExplorerFormsCollectionItem: TadxOlFormsCollectionItem;
begin
  result := adxOlFormsManager1.Items[0];
end;

function TAddInModule.GetInspectorFormsCollectionItem: TadxOlFormsCollectionItem;
begin
  result := adxOlFormsManager1.Items[1];
end;

procedure TAddInModule.ProcessInspectorButtonClick;
var
  activeInspector: Inspector;
begin
  ShowByInspectorCommandBarButtonClick := true;
  try
    activeInspector := OutlookApp.ActiveInspector();
    try
      if (activeInspector <> nil) then begin
        InspectorFormsCollectionItem.ApplyTo(activeInspector);
      end;
    finally
      activeInspector := nil;
    end;
  finally
    ShowByInspectorCommandBarButtonClick := false;
  end;
end;

procedure TAddInModule.adxCOMAddInModuleAddInBeginShutdown(
  Sender: TObject);
begin
  folders.Free;
  folders := nil;
end;

procedure TAddInModule.adxCOMAddInModuleError(const E: Exception;
  var Handled: Boolean);
begin
  Handled := true;
  Common.ShowMessage(Strings.AddinModuleOnError);
end;

procedure TAddInModule.adxOlFormsManager1Error(ASender: TObject;
  Args: TadxOlErrorEventArgs);
begin
  Args.Handled := true;
  Common.ShowMessage(Strings.FormsManagerOnError);
end;

procedure TAddInModule.adxOlFormsManager1ADXBeforeFormInstanceCreateEx(
  ASender: TObject; Args: TBeforeFormInstanceCreateEventArgs);
var
  isExplorerForm: boolean;
begin
  isExplorerForm := (Args.ExplorerObj <> nil); //or args.Item == ExplorerFormsCollectionItem
  if (isExplorerForm
      and (explorerMode = efdmInboxAndSubfoldersItem)
      and (not OutlookUtils.IsUnderFolder(folders.InboxFolder, Args.FolderObj))) then begin
    //Prevent form creating
    Args.Cancel := true;
  end;
end;

procedure TAddInModule.adxOlFormsManager1Initialize(ASender: TObject);
begin
  OutlookUtils.InitOutlookVersion(OutlookApp.DefaultInterface);
  folders := OutlookFolders.Create(OutlookApp.DefaultInterface);//See AddinModule_AddinBeginShutdown
  SetExplorerFormDisplayMode(efdmInboxOnlyItem, nil);
end;

procedure TAddInModule.adxOlInspectorCommandBar1Controls0Click(
  Sender: TObject);
begin
  ProcessInspectorButtonClick();
end;

initialization
  TadxFactory.Create(ComServer, TCoVisibilityD7, CLASS_CoVisibilityD7, TAddInModule);

end.
