// The sample contained herein is provided to you "AS IS".
//
// It is furnished by Add-in Express as a simple example and has not been
// thoroughly tested under all conditions. Add-in Express, therefore, cannot
// guarantee its reliability, serviceability or functionality.

unit uUtils;

interface

uses
  Windows, Dialogs, Outlook2000, Office2000, adxolFormsManager, ActiveX, Forms,
  SysUtils;

type
  ProjectConsts = class
  public
    class function DevCode: string;
    class function HowtoName: string;
    class function ProjectShortName: string;
    class function MessageCaption: string;
    class function HowtoProjectName: string;
    class function CommandBarName: string;
  end;

  Strings = class
    class function AddinModuleOnError: string;
    class function FormsManagerOnError: string;
    class function InboxOnlyItem: string;
    class function InboxAndSubfoldersItem: string;
    class function AllFolders: string;
    class function TheFormWillHide: string;
    class function OutlookSecurityGuardDuringAccessingFields: string;
    class function DoYouWantShowForm: string;
  end;

  Common = class
  public
    class function CompareADXStrings(str1: string; str2: string): boolean;
    class function IndexOfString(source: string; str: string): integer;
    class procedure ShowError(mes: string);
    class procedure ShowMessage(mes: string);
  end;

  OutlookPane = (
    olFolderList = 2,
    olNavigationPane = 4,
    olOutlookBar = 1,
    olPreview = 3,
    olToDoPane = 5);

  HostVersion = (vUnknown, v2000, v2002, v2003, v2007);

  ExplorerFormDisplayMode = (efdmUnknown, efdmInboxOnlyItem, efdmInboxAndSubfoldersItem);

  OutlookUtils = class
  public
    class procedure InitOutlookVersion(OutlookApp: OutlookApplication);
    class function OutlookVersion: HostVersion;
    class function RefreshCurrentFolderExecuting: boolean;
    class function ExistsExplorerAdvancedFormRegion(AExplorer: Explorer;
      ExplorerLayout: TadxOlExplorerLayout; var failureMessage: string): boolean;
    class procedure RefreshCurrentFolder(AExplorer: Explorer);
    class procedure ClearFolderWebViewProperties(AFolder: MAPIFolder);
    class procedure SetFolderWebViewProperties(Folder: MAPIFolder; WebViewURL: string; WebViewOn: boolean);
    class function IsExplorerPaneVisible(AExplorer: Explorer; OlPane: TOleEnum): boolean;
    class procedure SetExplorerFolder(AExplorer: Explorer; Folder: MAPIFolder);

    class function GetExplorerSelectionItem(AExplorer: Explorer): IDispatch;
    class procedure SetEnablePropertyOfButtonByFindInInspector(adxCommandBarButton: CommandBarButton;
      Enable: boolean; AInspector: Inspector);
    class function GetDefaultFolder(OutlookAppObj: _Application; DefaultFolder: TOleEnum): MAPIFolder;
    class function GetRootFolder(OutlookAppObj: _Application): MAPIFolder;
    class function IsUnderFolder(RootFolderObj: MAPIFolder; FolderObj: MAPIFolder): boolean;
    class function CompareFolders(FolderObj1: MAPIFolder; FolderObj2: MAPIFolder): boolean;
    class function GetParentFolder(FolderObj: MAPIFolder): MAPIFolder;
  end;

  ADXUtils = class
  private
  public
    class procedure SetExplorerLayout(Item: TadxOlFormsCollectionItem;
      explorerLayoutText: string; OutlookApp: _Application);
    class procedure SetInspectorLayout(Item: TadxOlFormsCollectionItem;
      inspectorLayoutText: string; OutlookApp: _Application);
    class function ConvertToADXExplorerLayout(explorerLayoutText: string): TadxOlExplorerLayout;
    class function ConvertToADXOlInspectorLayout(inspectorLayoutText: string): TadxolInspectorLayout;
    class function ExistsVisibleForm(Item: TadxOlFormsCollectionItem): boolean;
    class function ADXHTMLFileName: string;
    class function GetVisibleForm(Item: TadxOlFormsCollectionItem): TadxOlForm;
  end;

  OutlookFolders = class
  private
      FInboxFolder:  MAPIFolder;
      FRootFolder: MAPIFolder;
  public
    constructor Create(OutlookApp: _Application);
    property InboxFolder: MAPIFolder read FInboxFolder;
    property RootFolder: MAPIFolder read FRootFolder;
    function RootFoderName: string;
    function InboxFolderName: string;
    function InboxFullFolderName: string;
    function IsInboxFolder(Folder: MAPIFolder): boolean;
  end;

implementation

var
  OutlookUtilsOutlookVersion: HostVersion;
  OutlookUtilsRefreshCurrentFolderExecuting: boolean;



{ ProjectConsts }

class function ProjectConsts.CommandBarName: string;
begin
  result := HowtoProjectName;
end;

class function ProjectConsts.DevCode: string;
begin
  result := 'D7';
end;

class function ProjectConsts.HowtoName: string;
begin
  result := 'Controlling the Visibility of the Form';
end;

class function ProjectConsts.HowtoProjectName: string;
begin
  result := ProjectShortName + DevCode;
end;

class function ProjectConsts.MessageCaption: string;
begin
  result := HowtoName;
end;

class function ProjectConsts.ProjectShortName: string;
begin
  result := 'Visibility';
end;

{ Strings }

class function Strings.AddinModuleOnError: string;
begin
  result := 'An error has been handled in the AddinModule.OnError event';
end;

class function Strings.AllFolders: string;
begin
  result := '*';
end;

class function Strings.DoYouWantShowForm: string;
begin
  result := 'Do you want show TadxOlForm?';
end;

class function Strings.FormsManagerOnError: string;
begin
  result := 'An error has been handled in the ADXOlFormsManagger.OnError event';
end;

class function Strings.InboxAndSubfoldersItem: string;
begin
  result := 'Inbox and subfolders';
end;

class function Strings.InboxOnlyItem: string;
begin
  result := 'Inbox only'; 
end;

class function Strings.OutlookSecurityGuardDuringAccessingFields: string;
begin
  result := 'In Outlook 2000 SP1 and Outlook XP, accessing some fields will trigger Outlook Security Guard.'
    + #13#10 +  'For Outlook XP, this is a well-known bug.'
    + #13#10 +  'For Outlook 2000, this is by design.';
end;


class function Strings.TheFormWillHide: string;
begin
  result := 'This operation will to hide the form.'
    + #13#10 + 'To restore it, click the Restore TadxOlForm1 button in the command bar';
end;

{ Common }

class function Common.CompareADXStrings(str1, str2: string): boolean;
begin
  result := UpperCase(str1) = UpperCase(str2);
end;

class function Common.IndexOfString(source, str: string): integer;
begin
  result := Pos(str, source);
end;

class procedure Common.ShowError(mes: string);
begin
  MessageBox(0, PChar(mes), PChar(ProjectConsts.MessageCaption), MB_OK or MB_ICONERROR or MB_APPLMODAL);
end;

class procedure Common.ShowMessage(mes: string);
begin
  MessageBox(0, PChar(mes), PChar(ProjectConsts.MessageCaption), MB_OK or MB_APPLMODAL);
end;

{ OutlookUtils }

class procedure OutlookUtils.ClearFolderWebViewProperties(
  AFolder: MAPIFolder);
begin
  SetFolderWebViewProperties(AFolder, '', false);
end;

class function OutlookUtils.CompareFolders(FolderObj1,
  FolderObj2: MAPIFolder): boolean;
var
  FolderEntryID1, FolderEntryID2: string;
begin
  if ((FolderObj1 = nil) or (FolderObj2 = nil)) then begin
   result := false;
   Exit;
  end;
  try
    FolderEntryID1 := FolderObj1.EntryID;
    FolderEntryID2 := FolderObj2.EntryID;
    if ((FolderEntryID1 = '') or (FolderEntryID2 = '')) then begin
      result := false;
      Exit;
    end;
  except
    result := false;
    exit;
  end;
  result := Common.CompareADXStrings(FolderEntryID1, FolderEntryID2);
end;

class function OutlookUtils.ExistsExplorerAdvancedFormRegion(
  AExplorer: Explorer; ExplorerLayout: TadxOlExplorerLayout;
  var failureMessage: string): boolean;
begin
  result := false;
  case (ExplorerLayout) of
    elLeftSubpane,
    elRightSubpane,
    elTopSubpane,
    elBottomSubpane: begin
      result := true;
      exit;
    end;

    elWebViewPane: begin
      result := true;
      exit;
    end;

    elBottomNavigationPane: begin
      case (OutlookUtils.OutlookVersion) of
        v2000,
        v2002:
          result := OutlookUtils.IsExplorerPaneVisible(AExplorer, ord(olFolderList));
        v2003,
        v2007:
          result := OutlookUtils.IsExplorerPaneVisible(AExplorer, 4{olNavigationPane});
      end;

      if (not result) then begin
        case (OutlookUtils.OutlookVersion) of
          v2000,
          v2002:
            failureMessage := 'Please, enable the Folder List. See the View | Folder List menu.';
          v2003,
          v2007:
            failureMessage := 'Please, enable the Navigation Pane. See the View | Navigation Pane menu or Press Alt + F1.';
        end;
      end;
      exit;
    end;

    elBottomOutlookBar: begin
      case (OutlookUtils.OutlookVersion) of
        v2000,
        v2002: begin
            result := IsExplorerPaneVisible(AExplorer, ord(olOutlookBar));
            if (not result) then begin
              failureMessage := 'Please, enable the Outlook Bar. See the View | Outlook Bar menu.';
            end;
            exit;
          end;
        v2003,
        v2007: begin
            failureMessage := 'Outlook Bar is available in 2000 and XP versions only.';
            exit;
          end;
      else
        exit;
      end;
    end;

    elBottomTodoBar: begin
      case (OutlookUtils.OutlookVersion) of
        v2000,
        v2002,
        v2003:
          failureMessage := 'To-Do bar is available starting from version 2007.';
        v2007: begin
          result := IsExplorerPaneVisible(AExplorer, 5 {olToDoPane});
          if (not result) then begin
            failureMessage := 'Please, enable To-Do Pane. See the View | To-Do Bar menu or press Alt + F2.';
          end;
        end;
      else
        exit;
      end;
    end;

    elBottomReadingPane,
    elLeftReadingPane,
    elRightReadingPane,
    elTopReadingPane: begin
      result := IsExplorerPaneVisible(AExplorer, ord(olPreview));
      if (not result) then begin
        case (OutlookUtils.OutlookVersion) of
          v2000,
          v2002:
            failureMessage := 'Please, enable the Preview Pane. See the View | Preview Pane menu.';
          v2003,
          v2007:
            failureMessage := 'Please, enable the Reading Pane. See the View | Reading Pane menu.';
        end;
      end;
    end;
    else
      exit;
  end;
end;

class function OutlookUtils.GetDefaultFolder(OutlookAppObj: _Application;
  DefaultFolder: TOleEnum): MAPIFolder;
var
  NS: NameSpace;
begin
  result := nil;
  try
    NS := OutlookAppObj.GetNamespace('MAPI');
    result := NS.GetDefaultFolder(DefaultFolder);
  finally
    NS := nil;
  end;
end;

class function OutlookUtils.GetExplorerSelectionItem(
  AExplorer: Explorer): IDispatch;
var
  sel: Selection;
begin
  result := nil;
  sel := AExplorer.Selection;
  try
    if (sel.Count > 0) then begin
      result := sel.Item(1);
    end;
  finally
    sel := nil;
  end;
end;

class function OutlookUtils.GetParentFolder(
  FolderObj: MAPIFolder): MAPIFolder;
var
  parent: IDispatch;
  FolderGuid: TGuid;
begin
  result := nil;
  try
    try
      parent := FolderObj.Parent;
    except
      parent := nil;
    end;
    if parent <> nil then begin
      FolderGuid := StringToGUID('{00063006-0000-0000-C000-000000000046}');
      parent.QueryInterface(FolderGuid, result);
    end
    else begin
      Log.AddMessage('GetParentFolder: parent = nil');
    end;
  finally
    parent := nil;
  end;
end;

class function OutlookUtils.GetRootFolder(
  OutlookAppObj: _Application): MAPIFolder;
var
  InboxFolder: MAPIFolder;
begin
  InboxFolder := OutlookUtils.GetDefaultFolder(OutlookAppObj, olFolderInbox);
  result := OutlookUtils.GetParentFolder(InboxFolder)
end;

class procedure OutlookUtils.InitOutlookVersion(
  OutlookApp: _Application);
var
  hostVersion: string;
begin
  if (OutlookApp <> nil) then begin
      hostVersion := OutlookApp.Version;

      if (Common.IndexOfString(hostVersion, '9.0') >= 0) then begin
        OutlookUtilsOutlookVersion := v2000;
      end;

      if (Common.IndexOfString(hostVersion, '10.0') >= 0) then begin
        OutlookUtilsOutlookVersion := v2002;
      end;

      if (Common.IndexOfString(hostVersion, '11.0') >= 0) then begin
        OutlookUtilsOutlookVersion := v2003;
      end;

      if (Common.IndexOfString(hostVersion, '12.0') >= 0) then begin
        OutlookUtilsOutlookVersion := v2007;
      end;
  end;
end;

class function OutlookUtils.IsExplorerPaneVisible(AExplorer: Explorer;
  OlPane: TOleEnum): boolean;
begin
  result := AExplorer.IsPaneVisible(OlPane);
end;

class function OutlookUtils.IsUnderFolder(RootFolderObj,
  FolderObj: MAPIFolder): boolean;
var
  ParentFolderObj: MAPIFolder;
begin
  try
    result := false;
    if ((RootFolderObj = nil) or (FolderObj = nil)) then exit;

    if CompareFolders(FolderObj, RootFolderObj) then begin
      result := true;
      exit;
    end else begin
      ParentFolderObj := GetParentFolder(FolderObj);
      result := IsUnderFolder(RootFolderObj, ParentFolderObj);
      exit;
    end;
  finally
    ParentFolderObj := nil;
  end;
end;

class function OutlookUtils.OutlookVersion: HostVersion;
begin
  result := OutlookUtilsOutlookVersion;
end;

class procedure OutlookUtils.RefreshCurrentFolder(AExplorer: Explorer);
var
  OutlookApplication: _Application;
  currentFolder: MAPIFolder;
  nmSpace:  NameSpace;
  outboxFolder: MAPIFolder;
begin
  OutlookUtilsRefreshCurrentFolderExecuting := true;
  try
    OutlookApplication := AExplorer.Application;
    currentFolder := AExplorer.CurrentFolder;
    nmSpace := OutlookApplication.GetNamespace('MAPI');
    outboxFolder := nmSpace.GetDefaultFolder(olFolderOutbox);
    try
      OutlookUtils.SetExplorerFolder(AExplorer, outboxFolder);
      Application.ProcessMessages;
      OutlookUtils.SetExplorerFolder(AExplorer, currentFolder);
    finally
      nmSpace := nil;
      currentFolder := nil;
      outboxFolder := nil;
      OutlookApplication := nil;
    end;
  finally
      OutlookUtilsRefreshCurrentFolderExecuting := false;
  end;
end;

class function OutlookUtils.RefreshCurrentFolderExecuting: boolean;
begin
  result := OutlookUtilsRefreshCurrentFolderExecuting;
end;

class procedure OutlookUtils.SetEnablePropertyOfButtonByFindInInspector(
  adxCommandBarButton: CommandBarButton; Enable: boolean;
  AInspector: Inspector);
var
  cmdBars: CommandBars;
  cmdBarControl: CommandBarControl;
  cmdBarButton: CommandBarButton;
begin
  cmdBars := AInspector.CommandBars;
  try
    cmdBarControl := cmdBars.FindControl(1 (*msoControlButton*),
        adxCommandBarButton.Id,
        adxCommandBarButton.Tag,
        true);
    if (cmdBarControl <> nil) then begin
        //Note, this button will be enabled/disabled in ALL inspector windows when Word is set as the default e-mail editor.
        cmdBarButton := cmdBarControl as CommandBarButton;
        if cmdBarButton <> nil then begin
          cmdBarButton.Set_Enabled(Enable);
        end;
    end;
  finally
    cmdBars := nil;
    cmdBarButton := nil;
    cmdBarControl := nil;
  end;
end;

class procedure OutlookUtils.SetExplorerFolder(AExplorer: Explorer;
  Folder: MAPIFolder);
begin
  AExplorer.CurrentFolder := Folder;
end;

class procedure OutlookUtils.SetFolderWebViewProperties(Folder: MAPIFolder;
  WebViewURL: string; WebViewOn: boolean);
begin
  if (Folder <> nil) then begin
    Folder.WebViewURL := WebViewURL;
    Folder.WebViewOn := WebViewOn;
  end;
end;

{ ADXUtils }

class function ADXUtils.ADXHTMLFileName: string;
begin
  result := 'ADXOlFormGeneral.html';
end;

class function ADXUtils.ConvertToADXExplorerLayout(
  explorerLayoutText: string): TadxolExplorerLayout;
begin
  result := elUnknown;
  if explorerLayoutText = 'elUnknown'              then result := elUnknown;
  if explorerLayoutText = 'elTopSubpane'           then result := elTopSubpane;
  if explorerLayoutText = 'elBottomSubpane'        then result := elBottomSubpane;
  if explorerLayoutText = 'elRightSubpane'         then result := elRightSubpane;
  if explorerLayoutText = 'elWebViewPane'          then result := elWebViewPane;
  if explorerLayoutText = 'elLeftSubpane'          then result := elLeftSubpane;
  if explorerLayoutText = 'elBottomOutlookBar'     then result := elBottomOutlookBar;
  if explorerLayoutText = 'elBottomNavigationPane' then result := elBottomNavigationPane;
  if explorerLayoutText = 'elBottomTodoBar'        then result := elBottomTodoBar;
  if explorerLayoutText = 'elTopReadingPane'       then result := elTopReadingPane;
  if explorerLayoutText = 'elBottomReadingPane'    then result := elBottomReadingPane;
  if explorerLayoutText = 'elLeftReadingPane'      then result := elLeftReadingPane;
  if explorerLayoutText = 'elRightReadingPane'     then result := elRightReadingPane;
end;

class function ADXUtils.ConvertToADXOlInspectorLayout(
  inspectorLayoutText: string): TadxolInspectorLayout;
begin
  result := ilUnknown;
  if inspectorLayoutText = 'ilUnknown'              then result := ilUnknown;
  if inspectorLayoutText = 'ilTopSubpane'           then result := ilTopSubpane;
  if inspectorLayoutText = 'ilBottomSubpane'        then result := ilBottomSubpane;
  if inspectorLayoutText = 'ilRightSubpane'         then result := ilRightSubpane;
  if inspectorLayoutText = 'ilLeftSubpane'          then result := ilLeftSubpane;
end;

class function ADXUtils.ExistsVisibleForm(Item: TadxOlFormsCollectionItem): boolean;
var
  i: integer;
begin
  result := false;
  for i := 0 to Item.FormInstanceCount - 1 do begin
    if (Item.FormInstances[i].Visible) then begin
      result := true;
      break;
    end;
  end;
end;

class function ADXUtils.GetVisibleForm(
  Item: TadxOlFormsCollectionItem): TadxOlForm;
var
  i: integer;
begin
  result := nil;
  for i := 0 to Item.FormInstanceCount - 1 do begin
    if (Item.FormInstances[i].Visible) then begin
      result := Item.FormInstances[i];
      break;
    end;
  end;
end;

class procedure ADXUtils.SetExplorerLayout(Item: TadxOlFormsCollectionItem;
  explorerLayoutText: string; OutlookApp: _Application);
var
  expl: Explorer;
  newExplorerLayout: TadxOlExplorerLayout;
  failureMessage: string;
  currentFolder: MAPIFolder;
  NeedClearFolderWebViewProperties: boolean;
begin
  expl := OutlookApp.ActiveExplorer;
  try
    newExplorerLayout := ADXUtils.ConvertToADXExplorerLayout(explorerLayoutText);
    if (newExplorerLayout <> elUnknown) then begin
      failureMessage := '';
      if (not OutlookUtils.ExistsExplorerAdvancedFormRegion(expl, newExplorerLayout, failureMessage)) then begin
        Common.ShowMessage(failureMessage);
      end;
       Item.ExplorerLayout := newExplorerLayout;
       if (not ADXUtils.ExistsVisibleForm(Item)) then begin
        if (newExplorerLayout = elWebViewPane) then begin
          OutlookUtils.RefreshCurrentFolder(expl);
          Item.ApplyTo(expl);
        end
        else begin
            currentFolder := expl.CurrentFolder;
            try
                NeedClearFolderWebViewProperties :=
                    (currentFolder.WebViewOn and (Common.IndexOfString(currentFolder.WebViewURL, ADXHTMLFileName) >= 0));

                if (NeedClearFolderWebViewProperties) then begin
                  OutlookUtils.ClearFolderWebViewProperties(currentFolder);
                  OutlookUtils.RefreshCurrentFolder(expl);
                  Item.ApplyTo(expl);
                end
                else begin
                  Item.ApplyTo(expl);
                end;
            finally
              currentFolder := nil;
            end;
        end;
      end;
    end;
  finally
    expl := nil;
  end
end;

class procedure ADXUtils.SetInspectorLayout(
  Item: TadxOlFormsCollectionItem; inspectorLayoutText: string;
  OutlookApp: _Application);
var
  newInspectorLayout: TadxOlInspectorLayout;
  activeInspector: Inspector;
begin
  newInspectorLayout := ADXUtils.ConvertToADXOlInspectorLayout(inspectorLayoutText);
  if (newInspectorLayout <> ilUnknown) then begin
    Item.InspectorLayout := newInspectorLayout;
  end;
  if (not ADXUtils.ExistsVisibleForm(Item)) then begin
      activeInspector := OutlookApp.ActiveInspector;
      try
       if (activeInspector <> nil) then begin
           Item.ApplyTo(activeInspector);
       end;
      finally
        activeInspector := nil;
      end;
  end;
end;

{ OutlookFolders }

constructor OutlookFolders.Create(OutlookApp: _Application);
begin
  FInboxFolder := OutlookUtils.GetDefaultFolder(OutlookApp, olFolderInbox);
  FRootFolder := OutlookUtils.GetRootFolder(OutlookApp);
end;

function OutlookFolders.InboxFolderName: string;
begin
  result := '';
  if (FInboxFolder <> nil) then begin
    result := FInboxFolder.Name;
  end;
end;

function OutlookFolders.InboxFullFolderName: string;
begin
  result := '';
  if ((FRootFolder <> nil) and (FInboxFolder <> nil)) then begin
    result := self.RootFoderName + '\' + Self.InboxFolderName;
  end;
end;

function OutlookFolders.IsInboxFolder(Folder: MAPIFolder): boolean;
begin
  result := ((Folder <> nil) and (Folder.Name = Self.InboxFolderName));
end;

function OutlookFolders.RootFoderName: string;
begin
  result := '';
  if (FRootFolder <> nil) then begin
    result := FRootFolder.Name;
  end;
end;

end.
